/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "authorize.hh"
#include "buffer.hh"
#include <iostream>
#include <stdexcept>

using namespace sends;
using namespace std;

//======================================  Construct authoriztion stage
authorize::authorize(int nThread, int nTrans, int nBuffer)
    : seda_stage(nThread, nTrans), resource(nBuffer, 2048), timeout(300)
{
}

authorize::~authorize(void) {
}

const char*
authorize::stage_name(void) const {
    return "authorize";
}

//======================================  Process authorization data.
void
authorize::set_parameters(const stage_pars& p) {
    timeout = p._timeout;
    seda_stage::set_parameters(p);
}

//======================================  Process authorization data.
void
authorize::filter(transaction* trans) {
    buffer* buf = 0;
    trans->set_stage(100);

    try {
	//------------------------------  Complete handshake
	debug_msg(dlvl_meta, trans->ident(), "complete handshake");
	trans->ref_socket().handshake();

	//------------------------------  Get an input buffer
	debug_msg(dlvl_meta, trans->ident(), "setup");
	trans->ref_socket().set_timeout(timeout);
	buf = resource.get_buffer();

	//------------------------------  Authenticate user and get his privs.
	debug_msg(dlvl_meta, trans->ident(), "check authorize request");
	trans->read(*buf);
	if (buf->str() != "authorize") {
	    throw runtime_error("Invalid client interaction");
	}

	//------------------------------  Authenticate user and get his privs.
	debug_msg(dlvl_meta, trans->ident(), "perform authentication");
	if (trans->secure()) trans->put_response(DAQD_SASL);
	else                 trans->put_response(DAQD_OK);
	trans->authenticate(*buf);

	//------------------------------  Log the user and source
	string userid = trans->ref_user().user();
	string msg = "authorized user: ";
	msg += userid + " from IP: " + trans->ref_socket().peer_name();
	error_msg(elvl_log, *trans, msg);

	//------------------------------  Send transaction to the next stage.
	trans->set_stage(110);
	trans->ref_socket().set_timeout(-1);
	put_output(trans);
    } catch (exception& e) {
	trans->set_stage(120);
	reject(trans, e.what());
    }
    if (buf) {
	buf->free();
    }
}
